###################################################
### chunk number 1: preliminaries
###################################################
library("dcemri")
options(width=85)
require("minpack.lm")
require("bitops")


###################################################
### chunk number 2: avg152T1_LR_nifti
###################################################
mni <- read.img("avg152T1_LR_nifti.nii.gz")
dim(mni)


###################################################
### chunk number 3: figure1-png
###################################################
png(filename="avg152T1_LR_nifti.png", width=400, height=400)


###################################################
### chunk number 4: figure1-code
###################################################
X <- nrow(mni)
Y <- ncol(mni)
Z <- nsli(mni)
zrange <- range(mni)
par(mfrow=c(10,10), mar=rep(0,4))
for (z in 1:Z) {
  image(1:X, 1:Y, mni[,,z,1], zlim=zrange, col=grey(0:64/64),
        xlab="", ylab="", axes=FALSE)
}


###################################################
### chunk number 5: figure1-dev.off
###################################################
dev.off()


###################################################
### chunk number 6: avg152T1_RL_nifti
###################################################
mni <- read.img("avg152T1_RL_nifti.nii.gz")
dim(mni)


###################################################
### chunk number 7: figure2-png
###################################################
png(filename="avg152T1_RL_nifti.png", width=400, height=400)


###################################################
### chunk number 8: figure2-code
###################################################
X <- nrow(mni)
Y <- ncol(mni)
Z <- nsli(mni)
zrange <- range(mni)
par(mfrow=c(10,10), mar=rep(0,4))
for (z in 1:Z) {
  image(1:X, 1:Y, mni[,,z,1], zlim=zrange, col=grey(0:64/64),
        xlab="", ylab="", axes=FALSE)
}


###################################################
### chunk number 9: figure2-dev.off
###################################################
dev.off()


###################################################
### chunk number 10: filtered_func_data
###################################################
ffd <- read.img("filtered_func_data.nii.gz")
dim(ffd)


###################################################
### chunk number 11: figure3-png
###################################################
png(filename="ffd.png", width=400, height=400)


###################################################
### chunk number 12: figure3-code
###################################################
X <- nrow(ffd)
Y <- ncol(ffd)
Z <- nsli(ffd)
W <- ntim(ffd)
zrange <- range(ffd)
w <- 1
par(mfrow=c(5,5), mar=rep(0,4))
for (z in 1:Z) {
  image(1:X, 1:Y, ffd[,,z,w], zlim=zrange,
        col=grey(0:64/64), xlab="", ylab="", axes=FALSE)
}


###################################################
### chunk number 13: figure3-dev.off
###################################################
dev.off()


###################################################
### chunk number 14: zstat1
###################################################
zstat1 <- read.img("zstat1.nii.gz")
dim(zstat1)


###################################################
### chunk number 15: figure4-png
###################################################
png("ffd_zstat1.png", width=400, height=400)


###################################################
### chunk number 16: figure4-code
###################################################
par(mfrow=c(5,5), mar=rep(0,4))
w <- 1
for (z in 1:Z) {
  image(1:X, 1:Y, ffd[,,z,w], zlim=zrange, col=grey(0:64/64), 
        xlab="", ylab="", axes=FALSE)
  image(1:X, 1:Y, ifelse(abs(zstat1[,,z,1]) > 5, zstat1[,,z,1], NA),
        zlim=range(zstat1), col=hotmetal(), add=TRUE)
}


###################################################
### chunk number 17: figure4-dev.off
###################################################
dev.off()


###################################################
### chunk number 18: doubleanglemethod
###################################################
sdam60 <- read.img("SDAM_ep2d_60deg_26slc.nii.gz")[,,1:26,]
dim(sdam60)
sdam120 <- read.img("SDAM_ep2d_120deg_26slc.nii.gz")[,,1:26,]
dim(sdam120)
sdam.image <- rowMeans(dam(sdam60, sdam120, 60), dims=3)
mask <- rowSums(sdam60, dims=3) > 500
dim(mask)


###################################################
### chunk number 19: figure5-png
###################################################
png("sdam.png", width=400, height=400)


###################################################
### chunk number 20: figure5-code
###################################################
zi <- 10:13
w <- 5
SDAM <- read.img("SDAM_smooth.nii.gz")[,,1:26,,drop=TRUE]
par(mfrow=c(2,2), mar=rep(0,4))
for (z in zi) {
  image(sdam120[,,z,w], col=grey(0:128/128), xlab="", ylab="", axes=FALSE)
  image(ifelse(mask[,,z], SDAM[,,z], 0), zlim=c(.5,1.5),
        col=tim.colors(), add=TRUE)
}


###################################################
### chunk number 21: figure5-dev.off
###################################################
dev.off()


###################################################
### chunk number 22: t1estimation
###################################################
fnames <- c("fl3d_vibe-5deg","fl3d_vibe-10deg","fl3d_vibe-20deg",
            "fl3d_vibe-25deg","fl3d_vibe-15deg")
alpha <- c(5,10,20,25,15)
nangles <- length(alpha)
X <- Y <- 64
Z <- 26
flip <- fangles <- array(0, c(X,Y,Z,nangles))
for (w in 1:nangles) {
  vibe <- read.img(fnames[w])
  flip[,,,w] <- vibe
  fangles[,,,w] <- array(alpha[w], c(X,Y,Z))
}
TR <- 4.22 / 1000 # seconds
fanglesB1 <- fangles * array(SDAM, c(X,Y,Z,nangles))
zi <- 10:13
mask[,,(! 1:Z %in% zi)] <- FALSE
R1 <- R1.fast(flip, mask, fanglesB1, TR, verbose=TRUE)


###################################################
### chunk number 23: figure6-png
###################################################
png("t1_phantom.png", width=400, height=400)


###################################################
### chunk number 24: figure6-code
###################################################
par(mfrow=c(2,2), mar=rep(0,4))
for (z in zi) {
  image(drop(vibe)[,,z], zlim=c(0,1024), col=grey(0:64/64),
        xlab="", ylab="", axes=FALSE)
  image(1/drop(R1$R10)[,,z], zlim=c(0,2.5), col=hotmetal(), add=TRUE)
}


###################################################
### chunk number 25: figure6-dev.off
###################################################
dev.off()


###################################################
### chunk number 26: FSLmask
###################################################
pmask <- read.img("t1_phantom_mask.nii.gz")
pmask[,,c(1:24,26),1] <- pmask[,,25,1] # repeat masked slice (#25) for all slices
dim(pmask)


###################################################
### chunk number 27: figure7
###################################################
T1 <- c(.484,.350,1.07,.648,.456,1.07,.660,1.543,1.543,.353)
par(mfrow=c(1,1), mar=c(5,4,4,2)+.1)
boxplot(split(1/drop(R1$R10), as.factor(drop(pmask)))[-1], 
        ylim=c(0,2.5), xlab="Region of Interest", ylab="T1 (seconds)")
points(1:10, T1, bg=rainbow(10), pch=21, cex=2)


###################################################
### chunk number 28: buckley.aif
###################################################
data("buckley")
aifparams <- with(buckley, orton.exp.lm(time.min, input))
fit.aif <- with(aifparams, aif.orton.exp(buckley$time.min, AB, muB, AG, muG))


###################################################
### chunk number 29: figure8
###################################################
with(buckley, plot(time.min, input, type="l", lwd=2, xlab="Time (minutes)", 
                   ylab=""))
with(buckley, lines(time.min, fit.aif, lwd=2, col=2))
legend("topright", c("Simulated AIF", "Estimated AIF"), lwd=2, col=1:2)


###################################################
### chunk number 30: buckley.kinetic
###################################################
xi <- seq(5, 300, by=5)
img <- array(t(breast$data)[,xi], c(13,1,1,60))
time <- buckley$time.min[xi]
aif <- buckley$input[xi]
mask <- array(TRUE, dim(img)[1:3])
aifparams <- orton.exp.lm(time, aif)
fit <- dcemri.lm(img, time, mask, model="orton.exp",
                 aif="user", user=aifparams)


###################################################
### chunk number 31: figure9
###################################################
par(mfrow=c(4,4), mar=c(5,4,4,2)/1.25, mex=0.8)
for (x in 1:nrow(img)) {
  plot(time, img[x,1,1,], ylim=range(img), xlab="Time (minutes)",
       ylab="", main=paste("Series", x))
  kinparams <- with(fit, c(vp[x,1,1], ktrans[x,1,1], kep[x,1,1]))
  lines(time, model.orton.exp(time, aifparams[1:4], kinparams), 
        lwd=1.25, col=2)
}


